//
// Functions to manage the display positioning and clearing of text on the two supported displays - 5110, TFT
//

void displayInit(byte csPin, byte dcPin, byte rstPin, byte displayType, byte rotationPoint){
  #if debugLevelProcedureCallTree
    altSerial.println(F("displayInit"));
  #endif

  //Display Setup
  pinMode(csPin, OUTPUT);   
  pinMode(dcPin, OUTPUT);
  pinMode(rstPin, OUTPUT);

  lcd5110Display.begin(84, 48); 
  lcd5110Display.setContrast(62);

}

void displaySetScreenParms(boolean clearPage){
  #if debugLevelProcedureCallTree
    altSerial.println(F("displaySetScreenParms"));
  #endif
  if (clearPage == true) {           
    lcd5110Display.clear();
  }
}

void displayPrintText(String tftText, byte dispRowX, byte dispRowY, byte dispRowYPad, byte textSizeScaleX, byte textSizeScaleY, textPosition textPositionFormat) {
  #if debugLevelProcedureCallTree
    altSerial.println(F("displayPrintText"));
  #endif

  switch (textPositionFormat) {
    ///
    case (text_XCoord_YCoord): {
      lcd5110Display.setCursor(dispRowX, dispRowY);
      break;
    }
    ///
    case (text_XLeftJustified_YCoord): {
      lcd5110Display.setCursor(0, dispRowY);
      break;
    }
    case (text_XCenterJustified_YCoord): {
      lcd5110Display.setCursor((getLineXPosition(tftText.length(),textSizeScaleX)/2), dispRowY);
      break;
    }
    case (text_XRightJustified_YCoord): {
      lcd5110Display.setCursor(getLineXPosition(tftText.length(),textSizeScaleX), dispRowY);
      break;
    ///
    }
    case (text_XCoord_YLineJustified): {
      lcd5110Display.setCursor(dispRowX, dispRowY);
      break;
    }
    ///
    case (text_XLeftJustified_YLineJustified): {
      lcd5110Display.setCursor(0, dispRowY);
      break;
    }
    case (text_XCenterJustified_YLineJustified): {
      lcd5110Display.setCursor((getLineXPosition(tftText.length(),textSizeScaleX)/2), dispRowY);
      break;
    }
    case (text_XRightJustified_YLineJustified): {
      lcd5110Display.setCursor(getLineXPosition(tftText.length(),textSizeScaleX), dispRowY);
      break;
    }
  }
  lcd5110Display.print(tftText);
}

void displayClearChars (byte numBlankingChars, byte dispRowX, byte dispRowY, byte dispRowYPad, byte textSizeScaleX, byte textSizeScaleY, textPosition textPositionFormat) {
  #if debugLevelProcedureCallTree
    altSerial.println(F("displayClearChars"));
  #endif

  int startX;
  int startY;
  int rectW;
  int rectH;
  
  switch (textPositionFormat) {
    case (text_XCoord_YCoord): {
      startX = dispRowX;
      startY = dispRowY;
      break;
    }
    ///
    case (text_XLeftJustified_YCoord): {
      startX = 0;
      startY = dispRowY;
      break;
    }
    case (text_XCenterJustified_YCoord): {
      startX = (getLineXPosition(numBlankingChars, textSizeScaleX)/2);
      startY = dispRowY;
      break;
    }
    case (text_XRightJustified_YCoord): {
      startX = getLineXPosition(numBlankingChars, textSizeScaleX);
      startY = dispRowY;
      break;
    }
    ///
    case (text_XCoord_YLineJustified): {
      startX = dispRowX;
      startY = dispRowY;
      break;
    }
    ///
    case (text_XLeftJustified_YLineJustified): {
      startX = 0;
      startY = dispRowY;
      break;
    }
    case (text_XCenterJustified_YLineJustified): {
      startX = (getLineXPosition(numBlankingChars, textSizeScaleX)/2);
      startY = dispRowY;
      break;
    }
    case (text_XRightJustified_YLineJustified): {
      startX = getLineXPosition(numBlankingChars, textSizeScaleX);
      startY = dispRowY;
      break;
    }
  }
  
  byte i;
  byte posX = startX;
  for (i=1; i<=numBlankingChars; i++) { 
    lcd5110Display.setCursor(posX, startY);
    lcd5110Display.print(" ");
    posX = startX + (i * (dispTextWidth5110*textSizeScaleX));
  }
}


byte getLineXPosition(byte stringLen, byte textSizeScaleX) {
  byte xPos;
  xPos = (lcd5110Display.width() - (stringLen * (dispTextWidth5110*textSizeScaleX)));
  return xPos;
}

byte getLineYPosition(byte dispRowY, byte dispRowYPad, byte textSizeScaleY) {
  byte yPos;
  yPos = (dispBaseYPos + (dispRowY * (dispRowYPad+(dispTextHeight*textSizeScaleY))));
  return yPos;
}
