//
//Procedures to process the incoming GRBL message and parse it
//
void readGRBLDataStream() {
  #if debugLevelProcedureCallTree
    altSerial.println(F("readGRBLDataStream"));
  #endif

  char receivedChars[numChars];  // an array to store the received data
  String GRBLString = "";
  boolean newGRBLData = false;
  boolean badGRBLData = false;
  //Read the serial input 
  recvGRBLDataStream(&newGRBLData, receivedChars, &badGRBLData);
  //We have a message to process
  if ( (newGRBLData == true) && (badGRBLData == false) ){
    newGRBLData = false;
    GRBLString = String(receivedChars); //Cast and trim the buffer to a String type
    GRBLString.trim(); 
    if (GRBLString != ""){
      parseGRBLString(&GRBLString); //We have a String - parse it
      noGRBLdataConnectionCounter = 0; //Reset the counter
    }
  }
}

//
//
//
void recvGRBLDataStream(boolean *newGRBLData, char *receivedChars, boolean *badGRBLData) {
  #if debugLevelProcedureCallTree
    altSerial.println(F("recvGRBLDataStream"));
  #endif

  static byte index = 0;
  char endMarker = '\n';
  char dataByte;
  
  while (Serial.available() > 0 && *newGRBLData == false && *badGRBLData == false) {
    dataByte = Serial.read();
    if (dataByte != endMarker) {
        receivedChars[index] = dataByte;
        index++;
        if (index >= numChars) {
            index = numChars - 1;
            *badGRBLData = true;
            #if debuglevelGRBLBadData
            altSerial.println(F(""));
            altSerial.println(F(""));
            altSerial.println(F(""));
            altSerial.println(F("C--------------> BAD GRBL Data <---------------"));
            altSerial.print(F("CHAR - ")); altSerial.println(String(receivedChars));
            altSerial.println(F(""));
            altSerial.println(F(""));
            altSerial.println(F(""));
            #endif
        } 
    } else {
      receivedChars[index] = '\0'; // terminate the string
      index = 0;
      *newGRBLData = true;
      #if debuglevelGRBLData
      //altSerial.println(F("")); altSerial.print(F("Char - ")); altSerial.println(String(receivedChars));
      #endif
    }
  }
}

//
//
//
void parseGRBLString(String *GRBLString) {
  #if debugLevelProcedureCallTree
    altSerial.println(F("parseGRBLString"));
  #endif

  //Parsing Variables
  String okString = "ok";
  String ok2String = "<ok>";
  String grblString = "Grbl";
  String errorString = "error";
  String alarmString = "ALARM:";
  String holdString = "Hold";
  String homeString = "HOME";
  String fsString = "";
  String fString = "";
  String coordPositions = "";

  byte okCount = 0;
  byte grblStringLength = 0;
  byte markPos;
  int openBracketPos = 0;
  int openBracketPos2 = 0;
  int opensqBracketPos = 0;
  int closeBracketPos = 0;
  int delimeterPos = 0;
  int colonPos = 0;
  int mPos = 0;
  int wPos = 0;
  int fPos = 0;
  int fsPos = 0;
  int pnPos = 0;
  int wcoPos = 0;
  int ovPos = 0;
  int dollarPos = 0;
  boolean patternMatch = false;

  // Lets parse the string - note that indexOf returns -1 if not found
  grblStringLength = GRBLString->length();     //altSerial.print(F("1- ")); altSerial.println(grblStringLength);
  openBracketPos = GRBLString->indexOf('<');   //altSerial.print(F("2- ")); altSerial.println(openBracketPos);
  openBracketPos2 = GRBLString->indexOf('<', openBracketPos + 1);   //altSerial.print(F("2.1- ")); altSerial.println(openBracketPos2);
  closeBracketPos = GRBLString->indexOf('>');  //altSerial.print(F("3- ")); altSerial.println(closeBracketPos);
  opensqBracketPos = GRBLString->indexOf('['); //altSerial.print(F("4- ")); altSerial.println(opensqBracketPos);
  delimeterPos = GRBLString->indexOf('|');     //altSerial.print(F("5- ")); altSerial.println(delimeterPos);
  colonPos = GRBLString->indexOf(':');         //altSerial.print(F("6- ")); altSerial.println(delimeterPos);
  mPos = GRBLString->indexOf("MPos:");         //altSerial.print(F("7- ")); altSerial.println(mPos);
  wPos = GRBLString->indexOf("WPos:");         //altSerial.print(F("8- ")); altSerial.println(wPos);
  fPos = GRBLString->indexOf("F:");            //altSerial.print(F("9- ")); altSerial.println(fPos);
  fsPos = GRBLString->indexOf("FS:");          //altSerial.print(F("10- ")); altSerial.println(fsPos);
  pnPos = GRBLString->indexOf("Pn:");          //altSerial.print(F("11- ")); altSerial.println(pnPos);
  wcoPos = GRBLString->indexOf("WCO");         //altSerial.print(F("12- ")); altSerial.println(wcoPos);
  ovPos = GRBLString->indexOf("Ov:");          //altSerial.print(F("13- ")); altSerial.println(ovPos);
  dollarPos = GRBLString->indexOf("$");        //altSerial.print(F("14- ")); altSerial.println(dollarPos);
  
  //altSerial.println(F(" "));

  if ( (openBracketPos != notFoundInString) && (openBracketPos2 == notFoundInString) && (closeBracketPos != notFoundInString) ) { //We have a GRBL MESSAGE to process that is not corrupt ex: 
    //<Idle|MPos:0.000,0.000,0.000|FS:0,0|Pn:XYZRS|WCO:39.136,0.000,0.000>    
    //<Idle|MPos:0.000,0.000,0.000|FS:0,0|Ov:100,100,100>
    //<Idle|MPos:0.000,0.000,0.000|FS:0,0>

    #if debuglevelGRBLData
    altSerial.println(F(""));
    altSerial.println(*GRBLString); //Print the message to altSerial
    #endif
  
    patternMatch = true;

    #if debuglevelGRBLData
      if (bit_isfalse(sysFlags, SYSFLAGS_COORDMODE)) {
        altSerial.println(F(" MPos"));
      } else {
        altSerial.println(F(" WPos"));
      }
    #endif

    if(colonPos < delimeterPos) {markPos = colonPos; } else {markPos = delimeterPos;}
    //altSerial.print(F("State: ")); altSerial.println(GRBLString->substring(openBracketPos+1,markPos));
    setSysState(GRBLString->substring(openBracketPos+1,markPos));
    //altSerial.println(sysState, BIN);
    
    if (wcoPos > 0) {
      coordPositions = GRBLString->substring(wcoPos,closeBracketPos); //WCO:0.000,0.000,0.000
        //altSerial.print(F(" Coord Pos: ")); altSerial.println(coordPositions);
      coordPositions = coordPositions.substring(coordPositions.indexOf(':')+1); //-50.000,-80.000,-2.000
       //altSerial.print(F(" Coord Pos: ")); altSerial.println(coordPositions);       
      fwcoXPos = (coordPositions.substring(0,coordPositions.indexOf(','))).toFloat(); //0.000
        coordPositions = coordPositions.substring(coordPositions.indexOf(',')+1); //-80.000,-2.000     
      fwcoYPos = (coordPositions.substring(0,coordPositions.indexOf(','))).toFloat(); //0.000
        coordPositions = coordPositions.substring(coordPositions.indexOf(',')+1); //-2.000     
      fwcoZPos = String(coordPositions).toFloat(); //0.000      
      #if debuglevelGRBLData
        altSerial.print(F("  wcoX Position: ")); altSerial.println(fwcoXPos);
        altSerial.print(F("  wcoY Position: ")); altSerial.println(fwcoYPos);
        altSerial.print(F("  wcoZ Position: ")); altSerial.println(fwcoZPos);
      #endif
    }
    
    coordPositions = GRBLString->substring(delimeterPos+1,GRBLString->indexOf('|',delimeterPos+1)); //MPos:0.000,0.000,0.000
      //altSerial.print(F(" Coord Pos: ")); altSerial.println(coordPositions);
    coordPositions = coordPositions.substring(coordPositions.indexOf(':')+1); //0.000,0.000,0.000
      //altSerial.print(F(" Coord Pos: ")); altSerial.println(coordPositions);
    if (bit_isfalse(sysFlags, SYSFLAGS_COORDMODE)) { //MPos
      fxPos = (coordPositions.substring(0,coordPositions.indexOf(','))).toFloat(); //0.000
        coordPositions = coordPositions.substring(coordPositions.indexOf(',')+1);      
      fyPos = (coordPositions.substring(0,coordPositions.indexOf(','))).toFloat(); //0.000
        coordPositions = coordPositions.substring(coordPositions.indexOf(',')+1);      
      fzPos = String(coordPositions).toFloat(); //0.000    
    }

    if (bit_istrue(sysFlags, SYSFLAGS_COORDMODE)) { //WPos
      fxPos = (coordPositions.substring(0,coordPositions.indexOf(','))).toFloat(); //0.000
        coordPositions = coordPositions.substring(coordPositions.indexOf(',')+1);      
      fyPos = (coordPositions.substring(0,coordPositions.indexOf(','))).toFloat(); //0.000
        coordPositions = coordPositions.substring(coordPositions.indexOf(',')+1);      
      fzPos = String(coordPositions).toFloat(); //0.000          
      fxPos = fxPos - fwcoXPos;
      fyPos = fyPos - fwcoYPos;
      fzPos = fzPos - fwcoZPos;
    }
    
    #if debuglevelGRBLData
      altSerial.print(F("  X Position: ")); altSerial.println(fxPos);
      altSerial.print(F("  Y Position: ")); altSerial.println(fyPos);
      altSerial.print(F("  Z Position: ")); altSerial.println(fzPos);
    #endif
    
    //
    //The GRBL message string will only contain either F: or FS:
    //
    if (fPos != notFoundInString) { //The GRBL message contains a string like F:
      delimeterPos = GRBLString->indexOf('|', fPos); //Look for the delimeter starting from the fpos position
      if (delimeterPos == notFoundInString) { //No delimeter found - so use the close bracket delimeter
        fString = GRBLString->substring(fPos,closeBracketPos); //F:350
          //altSerial.print(F("F Str: ")); altSerial.println(fString);
        feedRate = (fString.substring(fString.indexOf(':')+1)).toInt(); //350
          //altSerial.print(F("Feed Rate: ")); altSerial.println(feedRate);
      } else { //Delimeter found in the string - use the delimeter to get the feed rate
        fString = GRBLString->substring(fPos,delimeterPos); //F:350
          //altSerial.print(F("F Str: ")); altSerial.println(fString);
        feedRate = (fString.substring(fString.indexOf(':')+1)).toInt(); //350
          //altSerial.print(F("Feed Rate: ")); altSerial.println(feedRate);      
      }
    }
    if (fsPos != notFoundInString) { //The GRBL message contains a string like FS:
      delimeterPos = GRBLString->indexOf('|', fsPos); //Look for the delimeter starting from the fsPos position
      if (delimeterPos == notFoundInString) { //No delimeter found - so use the close bracket delimeter
        fsString = GRBLString->substring(fsPos,closeBracketPos); //FS:350,1500
          //altSerial.print(F("FS Str: ")); altSerial.println(fsString);
        feedRate = (fsString.substring(fsString.indexOf(':')+1,fsString.indexOf(','))).toInt(); //350   
        spindleRate = (fsString.substring(fsString.indexOf(',')+1)).toInt(); //1500
          //altSerial.print(F("Feed: ")); altSerial.println(feedRate);
          //altSerial.print(F("Spindle: ")); altSerial.println(spindleRate);
      } else { //Delimeter found in the string - use the delimeter to get the feed rate
        fsString = GRBLString->substring(fsPos,delimeterPos); //FS:350,1500
          //altSerial.print(F("FS Str: ")); altSerial.println(fsString);
        feedRate = (fsString.substring(fsString.indexOf(':')+1,fsString.indexOf(','))).toInt(); //350   
        spindleRate = (fsString.substring(fsString.indexOf(',')+1)).toInt(); //1500
          //altSerial.print(F("Feed: ")); altSerial.println(feedRate);
          //altSerial.print(F("Spindle: ")); altSerial.println(spindleRate);     
      }
    }

    //
    //Extract the pin states
    //
    if (pnPos != notFoundInString) { //Pin state is in the message
      //altSerial.println(GRBLString); //Print the message to altSerial
      delimeterPos = GRBLString->indexOf('|', pnPos); //Look for the delimeter starting from the pnPos position
      if (delimeterPos == notFoundInString) { //No delimeter found - so use the close bracket delimeter
        pinStates = GRBLString->substring(pnPos,closeBracketPos); //Pn:XYZRS
          //altSerial.print(F("1P1: ")); altSerial.println(pinStates);
        pinStates = pinStates.substring(pinStates.indexOf(':')+1); //XYZRS
          //altSerial.print(F("1P2: ")); altSerial.println(pinStates);
      } else { //Delimeter found in the string - use the delimeter to get the pin states
        pinStates = GRBLString->substring(pnPos,delimeterPos); //Pn:XYZRS
          //altSerial.print(F("2P1: ")); altSerial.println(pinStates);
        pinStates = pinStates.substring(pinStates.indexOf(':')+1); //XYZRS
          //altSerial.print(F("2P2: ")); altSerial.println(pinStates);      
      }
      pinStatesLength = pinStates.length();
        //altSerial.print(F("PSL: ")); altSerial.print(pinStatesLength); altSerial.println(pinStates);
    } else {
      pinStates = ""; 
      pinStatesLength = 0;       
    }
   
    if (GRBLString->indexOf("ok") != notFoundInString) {
      //altSerial.println(F("found an ok"));
    }
    
  } else { //We have an alternate message to process
    
    //altSerial.print(F("Alt GRBL String - ")); altSerial.println(GRBLString); //Print the message to altSerial

    #if debuglevelGRBLBadData
    if ( (openBracketPos != notFoundInString) && ((openBracketPos2 != notFoundInString) || (closeBracketPos == notFoundInString)) ) {     
      altSerial.println(F(""));
      altSerial.println(F(""));
      altSerial.println(F(""));
      altSerial.println(F("P--------------> BAD GRBL Data <---------------"));
      altSerial.print(F("PARSE - ")); altSerial.println(*GRBLString);
      altSerial.println(F(""));
      altSerial.println(F(""));
      altSerial.println(F(""));            
    }
    #endif

    //
    //We just received an ALARM:# message - all GRBL data stops at this point
    //
    //altSerial.println(GRBLString.indexOf(alarmString));
    if (GRBLString->indexOf(alarmString) >= 0) {
      sysState = SYSSTATE_ALARM;
      bit_true(sysFlags,SYSFLAGS_MALARM); //Set the flag 
    }

    //
    //We just received an HOME message - all GRBL data stops at this point
    //
    //altSerial.println(GRBLString.indexOf(homeString));
    if (GRBLString->indexOf(homeString) >= 0) {
      sysState = SYSSTATE_HOMING;
      bit_true(sysFlags,SYSFLAGS_HOMEMSG); //Set the flag
    }
    
    if ((*GRBLString == okString) || (*GRBLString == ok2String)) {
      okCount++;
      patternMatch = true;
      if (okCount >= maxOks) {
        //altSerial.print("ok> "); 
        //altSerial.println("ok"); 
        //altSerial.println(" ");
        okCount = 0;
      }
    }
    if (GRBLString->indexOf(grblString)!= notFoundInString) {
      patternMatch = true;
      //altSerial.print("G> "); 
      //altSerial.println(*GRBLString); 
      //altSerial.println(" ");
    }
    if (GRBLString->indexOf(errorString) != notFoundInString) {
      patternMatch = true;
      //altSerial.print("E> "); 
      //altSerial.println(*GRBLString); 
      //altSerial.println(" ");
    }
    if (GRBLString->indexOf(opensqBracketPos) != notFoundInString) {
      patternMatch = true;
      //altSerial.print("[> "); 
      //altSerial.println(*GRBLString); 
      //altSerial.println(" ");
    }      
    if (GRBLString->indexOf(dollarPos) != notFoundInString) {
      patternMatch = true;
      //altSerial.print("$> "); 
      //altSerial.println(*GRBLString); 
      //altSerial.println(" ");
    }  
    if (patternMatch == false) {
      //altSerial.print("oth> "); 
      //altSerial.println(*GRBLString); 
      //altSerial.println(" ");
    }  
  }
  
  *GRBLString = " "; //Clear the message string for the next cycle
  
}

//
//
//
void processNoGRBLDataStream() {
  #if debugLevelProcedureCallTree
    altSerial.println(F("processNoGRBLDataStream"));
  #endif

  //Only increment the no connection counter if not already in an alarm state due to a machine alarm state triggered
  //or in an active homing cycle
  if (bit_isfalse(sysFlags,SYSFLAGS_MALARM) && bit_isfalse(sysFlags,SYSFLAGS_HOMEMSG)) {
    #if !testMode
    noGRBLdataConnectionCounter++; //Increment the no GRBL data counter
    if (noGRBLdataConnectionCounter > noGRBLdataCounterMax) {
      bit_true(sysFlags,SYSFLAGS_NCP); //Set the flag         
      noGRBLdataConnectionCounter = 0;
    }
    #endif
  }
}
