//
//Procedures to process the JoyStick inputs in the X and Y axis and the button press
//

void initJoyStickPins() {
  #if debugLevelProcedureCallTree
    altSerial.println(F("initJoyStickPins"));
  #endif

  //Joystick Setup - no need to define the analog input pins
  pinMode(SW_pin, INPUT);
  digitalWrite(SW_pin, HIGH);
}

//
//
//
void readJoystickInputs() {
  #if debugLevelProcedureCallTree
    altSerial.println(F("readJoystickInputs"));
  #endif

  int JSXaxisVal = 0; //Analog input read value
  int JSYaxisVal = 0; //Analog input read value
  byte JSSwitchVal = 1;
  
  JSSwitchVal = digitalRead(SW_pin);
  if (JSSwitchVal == swPressed) {
    bit_true(sysFlags,SYSFLAGS_JSPRESSED);
  }
  JSXaxisVal = map(analogRead(X_pin),minAnalogRange,maxAnalogRange,xAxisRight,xAxisLeft); //altSerial.print(F("X-axis: ")); altSerial.println(JSXaxisVal);
  JSYaxisVal = map(analogRead(Y_pin),minAnalogRange,maxAnalogRange,yAxisFront,yAxisBack); //altSerial.print(F("Y-axis: ")); altSerial.println(JSYaxisVal);
  
  determineJoyStickMovements(JSXaxisVal, JSYaxisVal);

  if (bit_istrue(sysFlags,SYSFLAGS_CMP)){ //CMP - change menu page flag
    bit_false(sysFlags,SYSFLAGS_CMP); //if we moved to the jogging page with the JS rather than a click - then skip sending the message the first time
  } else {
    //We are on the jogging page and the user wants to Jog the machine - process the jogging movement messages
    processJSJoggingMovements(JSXaxisVal, JSYaxisVal);
  }
}

//
//
//
void determineJoyStickMovements(int JSXaxisVal, int JSYaxisVal) {
  #if debugLevelProcedureCallTree
    altSerial.println(F("determineJoyStickMovements"));
  #endif

  //Process Joystick movement in the X axis Direction - Right - Left

  if (menuPageActive == Jogging_L1) {
    
    if ( (((lastXAxisDir != JSXaxisVal) && (JSXaxisVal != xAxisHold)) && (sysState & SYSSTATE_JOG)) || (( (lastYAxisDir != JSYaxisVal) && (JSYaxisVal != yAxisHold)) && (sysState & SYSSTATE_JOG)) ) { 
      #if debugLevelMachineMovement
      altSerial.println(F("Send XY Jog Cancel is TRUE"));
      #endif
      sendJogCancel = true;
    } else {
      sendJogCancel = false;
    }
  }
    
  if (JSXaxisVal != xAxisHold) {
    
    #if debugLevelMachineMovement
    altSerial.print(F("JS X axisVal-")); altSerial.print(JSXaxisVal); altSerial.print(F("  Last X-")); altSerial.println(lastXAxisDir);
    #endif
    bit_true(movFlags, MOVEFLAG_XYAXIS);
  
    if (JSXaxisVal == xAxisRight) {  
      lastXAxisDir = JSXaxisVal;          
      bit_true(movFlags, MOVEFLAG_JSR);
      #if debugLevelMachineMovement
      altSerial.println(F("X-DIR: R")); //altSerial.println(JSXaxisDir);
      #endif
      lastYAxisDir = 0;
    } 
    if (JSXaxisVal == xAxisLeft) {
      lastXAxisDir = JSXaxisVal;
      bit_true(movFlags, MOVEFLAG_JSL);
      #if debugLevelMachineMovement
      altSerial.println(F("X-DIR: L")); //altSerial.println(JSXaxisDir);
      #endif
      lastYAxisDir = 0;
    }
  } else { //No movement
    bit_false(movFlags, MOVEFLAG_JSR);
    bit_false(movFlags, MOVEFLAG_JSL);
  }
  
  //Process Joystick movement in the Y axis Direction - Front - Back
  if (JSYaxisVal != yAxisHold) {
    
    #if debugLevelMachineMovement
    altSerial.print(F("JS Y axisVal-")); altSerial.print(JSYaxisVal); altSerial.print(F("  Last Y-")); altSerial.println(lastYAxisDir);
    #endif
    bit_true(movFlags, MOVEFLAG_XYAXIS);
    
    if (JSYaxisVal == yAxisFront ) {   
      lastYAxisDir = JSYaxisVal;   
      bit_true(movFlags, MOVEFLAG_JSF);
      #if debugLevelMachineMovement
      altSerial.println(F("Y-DIR: F")); //altSerial.println(JSYaxisDir);
      #endif
      lastXAxisDir = 0;
    }
    if (JSYaxisVal == yAxisBack ) {
      lastYAxisDir = JSYaxisVal;
      bit_true(movFlags, MOVEFLAG_JSB);
      #if debugLevelMachineMovement
      altSerial.println(F("Y-DIR: B")); //altSerial.println(JSYaxisDir);
      #endif
      lastXAxisDir = 0;
    }
  } else { //No movement
    bit_false(movFlags, MOVEFLAG_JSF);
    bit_false(movFlags, MOVEFLAG_JSB);
  }
  
}

//
//Procedure to create the GRBL jog command syntax - movement command.  ex: $J=G91G21X10F100\n where:
//                                                                    - G91 - incremental movement
//                                                                    - G21 - millimeters 
//                                                                    - X10 - move on the positive X direction 10 millimeters (X-10 for negative direction)
//                                                                    - F100 - feed rate (speed of movement) is 100
//
void processJSJoggingMovements(int JSXaxisVal, int JSYaxisVal) {
  #if debugLevelProcedureCallTree
    altSerial.println(F("processJSJoggingMovements"));
  #endif

  if (menuPageActive == Jogging_L1) {  //Only allow if the Jogging page is active
    if (bit_istrue(movFlags, MOVEFLAG_XYAXIS)) { //Only create a movement command if the joystick has been moved in either the X or Y direction

      //Clear the Z axis flags
      bit_false(movFlags, MOVEFLAG_ROTU);
      bit_false(movFlags, MOVEFLAG_ROTD);
      bit_false(jogBlinkFlags, JOGBLINK_Z);
      lastZMovementDir = 0;

      //
      // If we have changed direction on the joystick then send a jog cancel
      if (sendJogCancel == true) {
        #if debugLevelMachineMovement
        altSerial.println(F("JS - Change Direction - Send Jog Cancel"));
        #endif
        sendCharMessageToGRBL(txTransmitOnOpenTimeShort, cmdJogCancel); //Cancel this direction movement as we are no longer going in this direction
      }

      if (JSXaxisVal != xAxisHold) {
        if (bit_istrue(movFlags, MOVEFLAG_JSL)) {
          movementCommandString = "$J=G91G21X-" + String(jogIncrement[jogIncrementIndex]) + "F" + String(feedIncrement[feedIncrementIndex]) + "\n";
          #if debugLevelMachineMovement
          altSerial.print(F("Move to Left - ")); altSerial.println(movementCommandString);
          #endif
          bit_true(jogBlinkFlags,JOGBLINK_L);

          bit_false(movFlags, MOVEFLAG_JSR);
          bit_false(movFlags, MOVEFLAG_JSB);
          bit_false(movFlags, MOVEFLAG_JSF);
          bit_false(jogBlinkFlags,JOGBLINK_R);
          bit_false(jogBlinkFlags,JOGBLINK_B);
          bit_false(jogBlinkFlags,JOGBLINK_F);
        }
        
        if (bit_istrue(movFlags, MOVEFLAG_JSR)) {
          movementCommandString = "$J=G91G21X" + String(jogIncrement[jogIncrementIndex]) + "F" + String(feedIncrement[feedIncrementIndex]) +"\n";
          #if debugLevelMachineMovement
          altSerial.print(F("Move to Right - ")); altSerial.println(movementCommandString); 
          #endif
          bit_true(jogBlinkFlags,JOGBLINK_R);                     

          bit_false(movFlags, MOVEFLAG_JSL);
          bit_false(movFlags, MOVEFLAG_JSB);
          bit_false(movFlags, MOVEFLAG_JSF);
          bit_false(jogBlinkFlags,JOGBLINK_L);
          bit_false(jogBlinkFlags,JOGBLINK_B);
          bit_false(jogBlinkFlags,JOGBLINK_F);
        } 
      }

      if (JSYaxisVal != yAxisHold ) {
        if (bit_istrue(movFlags, MOVEFLAG_JSF)) {
          movementCommandString = "$J=G91G21Y" + String(jogIncrement[jogIncrementIndex]) + "F" + String(feedIncrement[feedIncrementIndex]) +"\n";
          #if debugLevelMachineMovement
          altSerial.print(F("Move to Front - ")); altSerial.println(movementCommandString);
          #endif
          bit_true(jogBlinkFlags,JOGBLINK_F);

          bit_false(movFlags, MOVEFLAG_JSL);
          bit_false(movFlags, MOVEFLAG_JSR);
          bit_false(movFlags, MOVEFLAG_JSB);
          bit_false(jogBlinkFlags,JOGBLINK_L);
          bit_false(jogBlinkFlags,JOGBLINK_R);
          bit_false(jogBlinkFlags,JOGBLINK_B);
        } 
        
        if (bit_istrue(movFlags, MOVEFLAG_JSB)) {
          movementCommandString = "$J=G91G21Y-" + String(jogIncrement[jogIncrementIndex]) + "F" + String(feedIncrement[feedIncrementIndex]) +"\n";
          #if debugLevelMachineMovement
          altSerial.print(F("Move to Back - ")); altSerial.println(movementCommandString);   
          #endif
          bit_true(jogBlinkFlags,JOGBLINK_B);

          bit_false(movFlags, MOVEFLAG_JSL);
          bit_false(movFlags, MOVEFLAG_JSR);
          bit_false(movFlags, MOVEFLAG_JSF);
          bit_false(jogBlinkFlags,JOGBLINK_L);
          bit_false(jogBlinkFlags,JOGBLINK_R);
          bit_false(jogBlinkFlags,JOGBLINK_F);
        } 
      }  
    }
    bit_false(sysFlags,SYSFLAGS_CMP);       
  } else {
    //Clear any old movement commands
    movementCommandString = "";
  }
}
