//
//
//
void initGRBLPins() {
  #if debugLevelProcedureCallTree
    altSerial.println(F("initGRBLPins"));
  #endif

  //Interface Pins to GRBL
  pinMode(resetAbortInterfacePin, OUTPUT);
    digitalWrite(resetAbortInterfacePin, LOW);
  pinMode(feedHoldInterfacePin, OUTPUT);
    digitalWrite(feedHoldInterfacePin, LOW);
  pinMode(cycleStartResumeInterfacePin, OUTPUT);
    digitalWrite(cycleStartResumeInterfacePin, LOW); 
}

//
//
//
void initHWTxSwitchPin() {
  #if debugLevelProcedureCallTree
    altSerial.println(F("initHWTxSwitchPin"));
  #endif

  //Transmit enable pin 
  pinMode(txSwitchOnPin, OUTPUT);
  digitalWrite(txSwitchOnPin, txSwitchStateOff); //See chip defines - HW dependent ie: CD4066 or 74LS245 etc...
}

//
//Procedures to send messages to GRBL or signal GRBL interface pins
// Note: you cannot inject a message into the GRBL when the machine is in motion - this will cause data stream errors to GRBL and it will fault
//
void sendCharMessageToGRBL(int txTransmitOnTime, char charMessage) {
  #if debugLevelProcedureCallTree
    altSerial.println(F("sendCharMessageToGRBL"));
  #endif

  if (sysState == SYSSTATE_IDLE || (sysState & (SYSSTATE_JOG | SYSSTATE_ALARM | SYSSTATE_SLEEP))) {
    #if debugLevelMessageProcessing
    altSerial.print(F("-->Sending CHAR Command to GRBL - ")); altSerial.println(String(txTransmitOnTime)); altSerial.println(charMessage);
    #endif
    digitalWrite(txSwitchOnPin, txSwitchStateOn); //Enable serial data transmission to GRBL - assumes 74LS245 chip
    Serial.print(charMessage);
    delay(txTransmitOnTime); //Delay to allow serial information to be transmitted
    digitalWrite(txSwitchOnPin, txSwitchStateOff); //Disable serial data transmission to GRBL - assumes 74LS245 chip
  }
}

//
//
//
void sendMessageToGRBL(int txTransmitOnTime, String messageCommandString) {
  #if debugLevelProcedureCallTree
    altSerial.println(F("sendMessageToGRBL"));
  #endif

  if (sysState == SYSSTATE_IDLE || (sysState & (SYSSTATE_JOG | SYSSTATE_ALARM | SYSSTATE_SLEEP))) {
    #if debugLevelMessageProcessing
    altSerial.print(F("-->Sending Message Command to GRBL - ")); altSerial.println(String(txTransmitOnTime)); altSerial.println(messageCommandString);
    #endif
    digitalWrite(txSwitchOnPin, txSwitchStateOn); //Enable serial data transmission to GRBL - assumes 74LS245 chip
    Serial.print(messageCommandString);
    delay(txTransmitOnTime); //Delay to allow serial information to be transmitted
    digitalWrite(txSwitchOnPin, txSwitchStateOff); //Disable serial data transmission to GRBL - assumes 74LS245 chip
  }
}

//
//
//
void signalGRBLInterfacePin(int txPinOnTime, unsigned char interfacePin) {
  #if debugLevelProcedureCallTree
    altSerial.println(F("signalGRBLInterfacePin"));
  #endif
  #if debugLevelMessageProcessing
  altSerial.print(F("Signalling GRBL Pin - ")); altSerial.println(interfacePin);
  #endif
  digitalWrite(interfacePin, HIGH);
  delay(txPinOnTime); 
  digitalWrite(interfacePin, LOW);
}
