//
//
//
void initRotaryPins() {
  #if debugLevelProcedureCallTree
    altSerial.println(F("initRotaryPins"));
  #endif

  //Do not define the rotary pins as inputs - this is done in the Rotary library
  pinMode(rotarySwitch, INPUT); //Switch component of Rotary encoder
}

//
//
//
void readZRotaryInput(){
  #if debugLevelProcedureCallTree
    altSerial.println(F("readZRotaryInput"));
  #endif

  unsigned char zAxisMovementResult;
  // Check ZAxis rotary encoder.
  zAxisMovementResult = zAxis.process(); //This is the library call
  if (zAxisMovementResult != DIR_NONE) {//We have movement

    if (menuPageActive == Jogging_L1) {
      if ( (lastZMovementDir != zAxisMovementResult) && (sysState & SYSSTATE_JOG) ) {
        #if debugLevelMachineMovement
        altSerial.println(F("Send Z Jog Cancel is TRUE"));
        #endif        
        sendJogCancel = true;
      } else {
        sendJogCancel = false;
      }
    }
    
    bit_true(movFlags, MOVEFLAG_ZAXIS); // Set the Z movement flag
    
    if (zAxisMovementResult == DIR_CW){
      lastZMovementDir = zAxisMovementResult;
      bit_true(movFlags, MOVEFLAG_ROTU);
      #if debugLevelMachineMovement
      altSerial.println(F("Rotary CW - Z Move Up"));
      #endif
    }
    if (zAxisMovementResult == DIR_CCW){
      lastZMovementDir = zAxisMovementResult;
      bit_true(movFlags, MOVEFLAG_ROTD);
      #if debugLevelMachineMovement
      altSerial.println(F("Rotary CCW - Z Move Down"));
      #endif
    }
  } else { //No movement of the rotary
    bit_false(movFlags, MOVEFLAG_ZAXIS);
    bit_false(movFlags, MOVEFLAG_ROTU);
    bit_false(movFlags, MOVEFLAG_ROTD);
  }
}

//
//
//
void processZAxisMovement(){ //Procedure to create the command to jog the Z axis with a jog command - note FR is fixed at 50mm/min
  #if debugLevelProcedureCallTree
    altSerial.println(F("processZAxisMovement"));
  #endif

  if ((menuPageActive == Jogging_L1) || (menuPageActive == WPos_L1)) {  //Only allow if the Jogging or Work Position page is active
    if (bit_istrue(movFlags, MOVEFLAG_ZAXIS)) { //Only create a movement command if there is a rotation of the rotary encoder either CW or CCW

      //Clear the X and Y axis flags
      bit_false(movFlags, MOVEFLAG_JSL);
      bit_false(movFlags, MOVEFLAG_JSR);
      bit_false(movFlags, MOVEFLAG_JSF);
      bit_false(movFlags, MOVEFLAG_JSB);
      bit_false(jogBlinkFlags,JOGBLINK_L);
      bit_false(jogBlinkFlags,JOGBLINK_R);
      bit_false(jogBlinkFlags,JOGBLINK_F);
      bit_false(jogBlinkFlags,JOGBLINK_B);
      lastXAxisDir = 0;  
      lastYAxisDir = 0;

      //
      // If we have changed direction on the rotary encoder then send a jog cancel
      if (sendJogCancel == true) {
        #if debugLevelMachineMovement
        altSerial.println(F("Rotary - Change Direction - Send Jog Cancel"));
        #endif
        sendCharMessageToGRBL(txTransmitOnOpenTimeShort, cmdJogCancel); //Cancel this direction movement as we are no longer going in this direction
      }
            
      if (bit_istrue(movFlags, MOVEFLAG_ROTU)){
        movementZAxisCommandString = "$J=G91G21Z" + String(jogZAxisIncrement[jogZAxisIncrementIndex]) + "F" + String(feedZAxisIncrement[feedZAxisIncrementIndex]) + "\n";
        #if debugLevelMachineMovement
        altSerial.print(F("Z Up Command: ")); altSerial.println(movementZAxisCommandString);
        #endif
        bit_false(movFlags, MOVEFLAG_ROTD);
        bit_true(jogBlinkFlags, JOGBLINK_Z);
      } 
      if (bit_istrue(movFlags, MOVEFLAG_ROTD)){
        movementZAxisCommandString = "$J=G91G21Z-" + String(jogZAxisIncrement[jogZAxisIncrementIndex]) + "F" + String(feedZAxisIncrement[feedZAxisIncrementIndex]) + "\n";
        #if debugLevelMachineMovement
        altSerial.print(F("Z Dn Command: ")); altSerial.println(movementZAxisCommandString);
        #endif
        bit_false(movFlags, MOVEFLAG_ROTU);
        bit_true(jogBlinkFlags, JOGBLINK_Z);
      } 
    } else {
      //Clear any old Z axis movement commands
      movementZAxisCommandString = "";      
    }
    
  } else if (menuPageActive == Cutting_L1) {
    //When on the cutting page the rotary is used to select between WPOS and MPOS
    if (bit_istrue(movFlags, MOVEFLAG_ZAXIS)) {
      if (bit_istrue(movFlags, MOVEFLAG_ROTU)){
        bit_true(sysFlags, SYSFLAGS_COORDMODE); //WPos
      }
      if (bit_istrue(movFlags, MOVEFLAG_ROTD)){
        bit_false(sysFlags, SYSFLAGS_COORDMODE); //MPos
      }
    }
  }
}
