//
//
//
void manageStateMachine() {
  #if debugLevelProcedureCallTree
    altSerial.println(F("manageStateMachine"));
  #endif
  
  //Process the system state and flags set   
  if (millis() - ledPollingTimer > ledPollingPeriod) { //Only flash the LED based upon the LED polling period

    #if debugLevelSystemStateMessaging
    //altSerial.println("");
    printStateFlag(sysState, F("manageStateMachine - State"));
    #endif
    #if debuglevelSystemFlags
    //altSerial.println("");
    if (sysFlags != SYSFLAGS_NONE) {
      printSysFlags(sysFlags, F("manageStateMachine - Flags"));
    }
    #endif
    #if debugLevelMachineMovement
    //altSerial.println("");
    if (movFlags != MOVEFLAG_NONE) {
      printMovFlags(movFlags, F("manageStateMachine - Movement"));
    }
    #endif
    if (jogBlinkFlags != JOGBLINK_NONE) {
      //printBlinkFlags(jogBlinkFlags, F("manageStateMachine - Blink"));
    }
    
    #if debuglevelBitFlags
    //altSerial.println("");
    altSerial.print(F("sysState-")); altSerial.println(sysState, BIN);
    altSerial.print(F("sysFlags-")); altSerial.println(sysFlags, BIN);
    altSerial.print(F("movFlags-")); altSerial.println(movFlags, BIN);
    altSerial.print(F("jogBlinkFlags-")); altSerial.println(jogBlinkFlags, BIN);
    #endif

    switch (sysState) {
      case SYSSTATE_IDLE: {
        if ((bit_istrue(sysFlags,SYSFLAGS_NCP) || bit_istrue(sysFlags,SYSFLAGS_MALARM) || bit_istrue(sysFlags,SYSFLAGS_HOMEMSG)) && bit_istrue(sysFlags,SYSFLAGS_ALARMACK)) { //Alarm/Message has now been cleared - re-display the main menu
          bit_false(sysFlags,SYSFLAGS_NCP);
          bit_false(sysFlags,SYSFLAGS_MALARM);
          bit_false(sysFlags,SYSFLAGS_HOMEMSG);
          bit_false(sysFlags,SYSFLAGS_ALARMACK);
        }
        ledStandbyState();   
        break;
      }
      case SYSSTATE_ALARM: {
        ledAlarmClearState();             
        break;
      }
      case SYSSTATE_CHECK_MODE: {
        //Do nothing
        break;
      }
      case SYSSTATE_HOMING: {
        if (bit_isfalse(sysFlags,SYSFLAGS_ALARMACK)) { //Alarm not acknowledged
          if (menuPageActive != HOME_Alarm_L1) {
            displayHOMEAlarm(true, dispTextWrapF, colorDarkYellow); //Display the HOME Alarm Screen
          }
        }
        ledMovementState();        
        break;
      }
      case SYSSTATE_CYCLE: {
        ledMovementState();        
        break;
      }
      case SYSSTATE_HOLD: {
        ledHoldState();        
        break;
      }
      case SYSSTATE_JOG: {
        ledMovementState();        
        break;
      }
      case SYSSTATE_SAFETY_DOOR: {
        //Do nothing
        break;
      }
      case SYSSTATE_SLEEP: {
        ledSleepState();        
        break;
      }
    }

    if (bit_istrue(sysFlags,SYSFLAGS_MALARM)) { //ALARM: message from GRBL.
      #if debugLevelSystemStateMessaging
      altSerial.println(F("ALARM:"));
      #endif
      if (menuPageActive != MAS_Alarm_L1) {
        displayMASProblemAlarm(true, dispTextWrapF, colorDarkYellow); //Display the NCP Alarm Screen
      }
      ledAlarmState();
    }
    
    if (bit_istrue(sysFlags,SYSFLAGS_NCP)) { //No Connection Problem
      #if debugLevelSystemStateMessaging
      altSerial.println(F("NCP"));
      #endif
      if (menuPageActive != NCP_Alarm_L1) {
        displayNCPProblemAlarm(true, dispTextWrapF, colorDarkYellow); //Display the NCP Alarm Screen
      }
      ledAlarmState();
    }
    
    ledPollingTimer = millis(); //Reset the LED polling timer
  }
}

//
//
//
String returnStateString() {
  #if debugLevelProcedureCallTree
    altSerial.println(F("returnStateString"));
  #endif

  switch (sysState) {
    case SYSSTATE_IDLE: {
      return F("Idle");
      break;
    }
    case SYSSTATE_ALARM: {
      return F("Alarm");
      break;
    }
    case SYSSTATE_CHECK_MODE: {
      return F("Check");
      break;
    }
    case SYSSTATE_HOMING: {
      return F("Home");
      break;
    }
    case SYSSTATE_CYCLE: {
      return F("Run");
      break;
    }
    case SYSSTATE_HOLD: {
      return F("Hold");
      break;
    }
    case SYSSTATE_JOG: {
      return F("Jog");
      break;
    }
    case SYSSTATE_SAFETY_DOOR: {
      return F("Door");
      break;
    }
    case SYSSTATE_SLEEP: {
      return F("Sleep");
      break;
    }
    default: {
      return F("UNK?");
    }
  }
}

//
//
//
void setSysState(String GRBLStateString) {
  #if debugLevelProcedureCallTree
    altSerial.println(F("setSysState"));
  #endif

    //
    if (GRBLStateString == F("Idle")) {
        sysState = SYSSTATE_IDLE;
    }
    if (GRBLStateString == F("Run")) {
      sysState = SYSSTATE_CYCLE;
    }
    if (GRBLStateString == F("Jog")) {
      sysState = SYSSTATE_JOG;
    }
    if (GRBLStateString == F("Home")) {
      sysState = SYSSTATE_HOMING;
    }
    if (GRBLStateString == F("Alarm")) {
      sysState = SYSSTATE_ALARM;
    }
    if (GRBLStateString == F("Hold")) {
      sysState = SYSSTATE_HOLD;
    }
    if (GRBLStateString == F("Sleep")) {
      sysState = SYSSTATE_SLEEP;          
    }
}
